﻿using System.Linq;
using Ordering.API.Application.IntegrationCommands.Commands;

namespace Ordering.API.Application.IntegrationEvents.EventHandling
{
    using Microsoft.eShopOnContainers.BuildingBlocks.EventBus.Abstractions;
    using System.Threading.Tasks;
    using Events;
    using Microsoft.eShopOnContainers.Services.Ordering.Domain.AggregatesModel.OrderAggregate;

    public class OrderStockNotConfirmedIntegrationEventHandler : IIntegrationEventHandler<OrderStockNotConfirmedIntegrationEvent>
    {
        private readonly IOrderRepository _orderRepository;

        public OrderStockNotConfirmedIntegrationEventHandler(IOrderRepository orderRepository)
        {
            _orderRepository = orderRepository;
        }

        public async Task Handle(OrderStockNotConfirmedIntegrationEvent @event)
        {
            var orderToUpdate = await _orderRepository.GetWithDependenciesAsync(@event.OrderId);

            var orderStockNotConfirmedItems = @event.OrderStockItems
                .FindAll(c => !c.HasStock)
                .Select(c => c.ProductId);

            orderToUpdate.SetStockConfirmedStatus(orderStockNotConfirmedItems);

            await _orderRepository.UnitOfWork.SaveEntitiesAsync();
        }
    }
}